# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.boarding_link_api_response import BoardingLinkApiResponse
from ..types.orgid import Orgid
from ..types.payabli_api_response import PayabliApiResponse
from ..types.payabli_api_response_template_id import PayabliApiResponseTemplateId
from ..types.template_code import TemplateCode
from ..types.template_content import TemplateContent
from ..types.template_name import TemplateName
from ..types.template_query_record import TemplateQueryRecord
from ..types.template_query_response import TemplateQueryResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawTemplatesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def add_template(
        self,
        org_id_: int,
        *,
        org_id: typing.Optional[Orgid] = OMIT,
        pricing_id: typing.Optional[int] = OMIT,
        template_code: typing.Optional[TemplateCode] = OMIT,
        template_content: typing.Optional[TemplateContent] = OMIT,
        template_description: typing.Optional[str] = OMIT,
        template_name: typing.Optional[TemplateName] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponseTemplateId]:
        """
        Creates a boarding template in an organization.

        Parameters
        ----------
        org_id_ : int
            The numeric identifier for organization, assigned by Payabli.

        org_id : typing.Optional[Orgid]
            The ID of the organization the template belongs to.

        pricing_id : typing.Optional[int]

        template_code : typing.Optional[TemplateCode]

        template_content : typing.Optional[TemplateContent]

        template_description : typing.Optional[str]
            A description for the template.

        template_name : typing.Optional[TemplateName]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseTemplateId]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Templates/{jsonable_encoder(org_id_)}",
            method="POST",
            json={
                "orgId": org_id,
                "pricingId": pricing_id,
                "templateCode": template_code,
                "templateContent": convert_and_respect_annotation_metadata(
                    object_=template_content, annotation=TemplateContent, direction="write"
                ),
                "templateDescription": template_description,
                "templateName": template_name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseTemplateId,
                    parse_obj_as(
                        type_=PayabliApiResponseTemplateId,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_template(
        self, template_id: float, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayabliApiResponseTemplateId]:
        """
        Deletes a template by ID.

        Parameters
        ----------
        template_id : float
            The boarding template ID. Can be found at the end of the boarding template URL in PartnerHub. Example: `https://partner-sandbox.payabli.com/myorganization/boarding/edittemplate/80`. Here, the template ID is `80`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseTemplateId]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Templates/{jsonable_encoder(template_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseTemplateId,
                    parse_obj_as(
                        type_=PayabliApiResponseTemplateId,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def getlink_template(
        self, ignore_empty: bool, template_id: float, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[BoardingLinkApiResponse]:
        """
        Generates a boarding link from a boarding template.

        Parameters
        ----------
        ignore_empty : bool
            Ignore read-only and empty fields Default is `false`. If `ignoreEmpty` = `false` and any field is empty, then the request returns a failure response. If `ignoreEmpty` = `true`, the request returns the boarding link name regardless of whether fields are empty.

        template_id : float
            The boarding template ID. Can be found at the end of the boarding template URL in PartnerHub. Example: `https://partner-sandbox.payabli.com/myorganization/boarding/edittemplate/80`. Here, the template ID is `80`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BoardingLinkApiResponse]
            Success.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Templates/getlink/{jsonable_encoder(template_id)}/{jsonable_encoder(ignore_empty)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BoardingLinkApiResponse,
                    parse_obj_as(
                        type_=BoardingLinkApiResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_template(
        self, template_id: float, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[TemplateQueryRecord]:
        """
        Retrieves a boarding template's details by ID.

        Parameters
        ----------
        template_id : float
            The boarding template ID. Can be found at the end of the boarding template URL in PartnerHub. Example: `https://partner-sandbox.payabli.com/myorganization/boarding/edittemplate/80`. Here, the template ID is `80`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TemplateQueryRecord]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Templates/get/{jsonable_encoder(template_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TemplateQueryRecord,
                    parse_obj_as(
                        type_=TemplateQueryRecord,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_templates(
        self,
        org_id: int,
        *,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TemplateQueryResponse]:
        """
        Retrieves a list of boarding templates for an organization. Use filters to limit results. You can't make a request that includes filters from the API console in the documentation. The response won't be filtered. Instead, copy the request, remove `parameters=` and run the request in a different client.

        Parameters
        ----------
        org_id : int
            The numeric identifier for organization, assigned by Payabli.

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]

            Collection of field names, conditions, and values used to filter the query.

            <Info>
              **You must remove `parameters=` from the request before you send it, otherwise Payabli will ignore the filters.**

              Because of a technical limitation, you can't make a request that includes filters from the API console on this page. The response won't be filtered. Instead, copy the request, remove `parameters=` and run the request in a different client.

              For example:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?parameters=totalAmount(gt)=1000&limitRecord=20

              should become:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?totalAmount(gt)=1000&limitRecord=20
            </Info>


            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:
            - `createdAt` (gt, ge, lt, le, eq, ne)
            - `title` (ct, nct)
            - `description` (ct, nct)
            - `code` (ct, nct)
            - `orgParentname` (ct, nct)

            List of comparison accepted - enclosed between parentheses:
            - eq or empty => equal
            - gt => greater than
            - ge => greater or equal
            - lt => less than
            - le => less or equal
            - ne => not equal
            - ct => contains
            - nct => not contains
            - in => inside array
            - nin => not inside array

            List of parameters accepted:
            - limitRecord : max number of records for query (default="20", "0" or negative value for all)
            - fromRecord : initial record in query

            Example: title(ct)=hoa return all records with title containing "hoa"

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TemplateQueryResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Query/templates/{jsonable_encoder(org_id)}",
            method="GET",
            params={
                "fromRecord": from_record,
                "limitRecord": limit_record,
                "parameters": parameters,
                "sortBy": sort_by,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TemplateQueryResponse,
                    parse_obj_as(
                        type_=TemplateQueryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_template(
        self,
        template_id: float,
        *,
        org_id: typing.Optional[Orgid] = OMIT,
        pricing_id: typing.Optional[int] = OMIT,
        template_code: typing.Optional[TemplateCode] = OMIT,
        template_content: typing.Optional[TemplateContent] = OMIT,
        template_description: typing.Optional[str] = OMIT,
        template_name: typing.Optional[TemplateName] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponseTemplateId]:
        """
        Updates a boarding template by ID.

        Parameters
        ----------
        template_id : float
            The boarding template ID. Can be found at the end of the boarding template URL in PartnerHub. Example: `https://partner-sandbox.payabli.com/myorganization/boarding/edittemplate/80`. Here, the template ID is `80`.

        org_id : typing.Optional[Orgid]
            The ID of the organization the template belongs to.

        pricing_id : typing.Optional[int]

        template_code : typing.Optional[TemplateCode]

        template_content : typing.Optional[TemplateContent]

        template_description : typing.Optional[str]
            A description for the template.

        template_name : typing.Optional[TemplateName]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseTemplateId]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Templates/{jsonable_encoder(template_id)}",
            method="PUT",
            json={
                "orgId": org_id,
                "pricingId": pricing_id,
                "templateCode": template_code,
                "templateContent": convert_and_respect_annotation_metadata(
                    object_=template_content, annotation=TemplateContent, direction="write"
                ),
                "templateDescription": template_description,
                "templateName": template_name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseTemplateId,
                    parse_obj_as(
                        type_=PayabliApiResponseTemplateId,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawTemplatesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def add_template(
        self,
        org_id_: int,
        *,
        org_id: typing.Optional[Orgid] = OMIT,
        pricing_id: typing.Optional[int] = OMIT,
        template_code: typing.Optional[TemplateCode] = OMIT,
        template_content: typing.Optional[TemplateContent] = OMIT,
        template_description: typing.Optional[str] = OMIT,
        template_name: typing.Optional[TemplateName] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponseTemplateId]:
        """
        Creates a boarding template in an organization.

        Parameters
        ----------
        org_id_ : int
            The numeric identifier for organization, assigned by Payabli.

        org_id : typing.Optional[Orgid]
            The ID of the organization the template belongs to.

        pricing_id : typing.Optional[int]

        template_code : typing.Optional[TemplateCode]

        template_content : typing.Optional[TemplateContent]

        template_description : typing.Optional[str]
            A description for the template.

        template_name : typing.Optional[TemplateName]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseTemplateId]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Templates/{jsonable_encoder(org_id_)}",
            method="POST",
            json={
                "orgId": org_id,
                "pricingId": pricing_id,
                "templateCode": template_code,
                "templateContent": convert_and_respect_annotation_metadata(
                    object_=template_content, annotation=TemplateContent, direction="write"
                ),
                "templateDescription": template_description,
                "templateName": template_name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseTemplateId,
                    parse_obj_as(
                        type_=PayabliApiResponseTemplateId,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_template(
        self, template_id: float, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayabliApiResponseTemplateId]:
        """
        Deletes a template by ID.

        Parameters
        ----------
        template_id : float
            The boarding template ID. Can be found at the end of the boarding template URL in PartnerHub. Example: `https://partner-sandbox.payabli.com/myorganization/boarding/edittemplate/80`. Here, the template ID is `80`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseTemplateId]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Templates/{jsonable_encoder(template_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseTemplateId,
                    parse_obj_as(
                        type_=PayabliApiResponseTemplateId,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def getlink_template(
        self, ignore_empty: bool, template_id: float, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[BoardingLinkApiResponse]:
        """
        Generates a boarding link from a boarding template.

        Parameters
        ----------
        ignore_empty : bool
            Ignore read-only and empty fields Default is `false`. If `ignoreEmpty` = `false` and any field is empty, then the request returns a failure response. If `ignoreEmpty` = `true`, the request returns the boarding link name regardless of whether fields are empty.

        template_id : float
            The boarding template ID. Can be found at the end of the boarding template URL in PartnerHub. Example: `https://partner-sandbox.payabli.com/myorganization/boarding/edittemplate/80`. Here, the template ID is `80`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BoardingLinkApiResponse]
            Success.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Templates/getlink/{jsonable_encoder(template_id)}/{jsonable_encoder(ignore_empty)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BoardingLinkApiResponse,
                    parse_obj_as(
                        type_=BoardingLinkApiResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_template(
        self, template_id: float, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[TemplateQueryRecord]:
        """
        Retrieves a boarding template's details by ID.

        Parameters
        ----------
        template_id : float
            The boarding template ID. Can be found at the end of the boarding template URL in PartnerHub. Example: `https://partner-sandbox.payabli.com/myorganization/boarding/edittemplate/80`. Here, the template ID is `80`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TemplateQueryRecord]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Templates/get/{jsonable_encoder(template_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TemplateQueryRecord,
                    parse_obj_as(
                        type_=TemplateQueryRecord,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_templates(
        self,
        org_id: int,
        *,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TemplateQueryResponse]:
        """
        Retrieves a list of boarding templates for an organization. Use filters to limit results. You can't make a request that includes filters from the API console in the documentation. The response won't be filtered. Instead, copy the request, remove `parameters=` and run the request in a different client.

        Parameters
        ----------
        org_id : int
            The numeric identifier for organization, assigned by Payabli.

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]

            Collection of field names, conditions, and values used to filter the query.

            <Info>
              **You must remove `parameters=` from the request before you send it, otherwise Payabli will ignore the filters.**

              Because of a technical limitation, you can't make a request that includes filters from the API console on this page. The response won't be filtered. Instead, copy the request, remove `parameters=` and run the request in a different client.

              For example:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?parameters=totalAmount(gt)=1000&limitRecord=20

              should become:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?totalAmount(gt)=1000&limitRecord=20
            </Info>


            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:
            - `createdAt` (gt, ge, lt, le, eq, ne)
            - `title` (ct, nct)
            - `description` (ct, nct)
            - `code` (ct, nct)
            - `orgParentname` (ct, nct)

            List of comparison accepted - enclosed between parentheses:
            - eq or empty => equal
            - gt => greater than
            - ge => greater or equal
            - lt => less than
            - le => less or equal
            - ne => not equal
            - ct => contains
            - nct => not contains
            - in => inside array
            - nin => not inside array

            List of parameters accepted:
            - limitRecord : max number of records for query (default="20", "0" or negative value for all)
            - fromRecord : initial record in query

            Example: title(ct)=hoa return all records with title containing "hoa"

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TemplateQueryResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Query/templates/{jsonable_encoder(org_id)}",
            method="GET",
            params={
                "fromRecord": from_record,
                "limitRecord": limit_record,
                "parameters": parameters,
                "sortBy": sort_by,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TemplateQueryResponse,
                    parse_obj_as(
                        type_=TemplateQueryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_template(
        self,
        template_id: float,
        *,
        org_id: typing.Optional[Orgid] = OMIT,
        pricing_id: typing.Optional[int] = OMIT,
        template_code: typing.Optional[TemplateCode] = OMIT,
        template_content: typing.Optional[TemplateContent] = OMIT,
        template_description: typing.Optional[str] = OMIT,
        template_name: typing.Optional[TemplateName] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponseTemplateId]:
        """
        Updates a boarding template by ID.

        Parameters
        ----------
        template_id : float
            The boarding template ID. Can be found at the end of the boarding template URL in PartnerHub. Example: `https://partner-sandbox.payabli.com/myorganization/boarding/edittemplate/80`. Here, the template ID is `80`.

        org_id : typing.Optional[Orgid]
            The ID of the organization the template belongs to.

        pricing_id : typing.Optional[int]

        template_code : typing.Optional[TemplateCode]

        template_content : typing.Optional[TemplateContent]

        template_description : typing.Optional[str]
            A description for the template.

        template_name : typing.Optional[TemplateName]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseTemplateId]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Templates/{jsonable_encoder(template_id)}",
            method="PUT",
            json={
                "orgId": org_id,
                "pricingId": pricing_id,
                "templateCode": template_code,
                "templateContent": convert_and_respect_annotation_metadata(
                    object_=template_content, annotation=TemplateContent, direction="write"
                ),
                "templateDescription": template_description,
                "templateName": template_name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseTemplateId,
                    parse_obj_as(
                        type_=PayabliApiResponseTemplateId,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class NotificationLogSearchRequest(UniversalBaseModel):
    start_date: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="startDate")] = pydantic.Field()
    """
    The start date for the search.
    """

    end_date: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="endDate")] = pydantic.Field()
    """
    The end date for the search.
    """

    notification_event: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="notificationEvent")] = (
        pydantic.Field(default=None)
    )
    """
    The type of notification event to filter by.
    """

    succeeded: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether the notification was successful.
    """

    org_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="orgId")] = pydantic.Field(
        default=None
    )
    """
    The ID of the organization to filter by.
    """

    paypoint_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="paypointId")] = pydantic.Field(
        default=None
    )
    """
    The ID of the paypoint to filter by.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class PayabliApiResponseError400ResponseData(UniversalBaseModel):
    """
    Describes the reason for a failed operation and how to resolve it.
    """

    explanation: typing.Optional[str] = pydantic.Field(default=None)
    """
    Describes the reason the operation failed.
    """

    todo_action: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="todoAction")] = pydantic.Field(
        default=None
    )
    """
    Describes how to resolve the error.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class QueryBatchesTransfer(UniversalBaseModel):
    """
    Transfer details within a batch response.
    """

    transfer_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="TransferId")] = pydantic.Field(
        default=None
    )
    """
    The transfer ID.
    """

    transfer_date: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="TransferDate")] = (
        pydantic.Field(default=None)
    )
    """
    The transfer date.
    """

    processor: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Processor")] = pydantic.Field(
        default=None
    )
    """
    The processor used for the transfer.
    """

    transfer_status: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="TransferStatus")] = (
        pydantic.Field(default=None)
    )
    """
    The transfer status.
    """

    gross_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="GrossAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The gross amount of the transfer.
    """

    charge_back_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="ChargeBackAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The chargeback amount.
    """

    returned_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="ReturnedAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The returned amount.
    """

    refund_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="RefundAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The refund amount.
    """

    hold_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="HoldAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The amount being held.
    """

    released_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="ReleasedAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The amount that has been released.
    """

    billing_fees_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="BillingFeesAmount")
    ] = pydantic.Field(default=None)
    """
    The billing fees amount.
    """

    third_party_paid_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="ThirdPartyPaidAmount")
    ] = pydantic.Field(default=None)
    """
    The third party paid amount.
    """

    adjustments_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="AdjustmentsAmount")
    ] = pydantic.Field(default=None)
    """
    The adjustments amount.
    """

    net_funded_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="NetFundedAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The net funded amount.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.payabli_api_response import PayabliApiResponse
from .types.stat_basic_query_record import StatBasicQueryRecord
from .types.statistics_vendor_query_record import StatisticsVendorQueryRecord
from .types.subscription_stats_query_record import SubscriptionStatsQueryRecord


class RawStatisticClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def basic_stats(
        self,
        entry_id: int,
        freq: str,
        level: int,
        mode: str,
        *,
        end_date: typing.Optional[str] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        start_date: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.List[StatBasicQueryRecord]]:
        """
        Retrieves the basic statistics for an organization or a paypoint, for a given time period, grouped by a particular frequency.

        Parameters
        ----------
        entry_id : int
            Identifier in Payabli for the entity.

        freq : str
            Frequency to group series. Allowed values:

            - `m` - monthly
            - `w` - weekly
            - `d` - daily
            - `h` - hourly

            For example, `w` groups the results by week.

        level : int
            The entry level for the request:
              - 0 for Organization
              - 2 for Paypoint

        mode : str
            Mode for the request. Allowed values:

            - `custom` - Allows you to set a custom date range
            - `ytd` - Year To Date
            - `mtd` - Month To Date
            - `wtd` - Week To Date
            - `today` - All current day
            - `m12` - Last 12 months
            - `d30` - Last 30 days
            - `h24` - Last 24 hours
            - `lasty` - Last Year
            - `lastm` - Last Month
            - `lastw` - Last Week
            - `yesterday` - Last Day


        end_date : typing.Optional[str]
            Used with `custom` mode. The end date for the range.
            Valid formats:
              - YYYY-mm-dd
              - YYYY/mm/dd
              - mm-dd-YYYY
              - mm/dd/YYYY

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            List of parameters.

        start_date : typing.Optional[str]
            Used with `custom` mode. The start date for the range.
            Valid formats:
               - YYYY-mm-dd
               - YYYY/mm/dd
               -  mm-dd-YYYY
               - mm/dd/YYYY

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[StatBasicQueryRecord]]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Statistic/basic/{jsonable_encoder(mode)}/{jsonable_encoder(freq)}/{jsonable_encoder(level)}/{jsonable_encoder(entry_id)}",
            method="GET",
            params={
                "endDate": end_date,
                "parameters": parameters,
                "startDate": start_date,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[StatBasicQueryRecord],
                    parse_obj_as(
                        type_=typing.List[StatBasicQueryRecord],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def customer_basic_stats(
        self,
        customer_id: int,
        freq: str,
        mode: str,
        *,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.List[SubscriptionStatsQueryRecord]]:
        """
        Retrieves the basic statistics for a customer for a specific time period, grouped by a selected frequency.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        freq : str
            Frequency to group series. Allowed values:

            - `m` - monthly
            - `w` - weekly
            - `d` - daily
            - `h` - hourly

            For example, `w` groups the results by week.

        mode : str
            Mode for request. Allowed values:

            - `ytd` - Year To Date
            - `mtd` - Month To Date
            - `wtd` - Week To Date
            - `today` - All current day
            - `m12` - Last 12 months
            - `d30` - Last 30 days
            - `h24` - Last 24 hours
            - `lasty` - Last Year
            - `lastm` - Last Month
            - `lastw` - Last Week
            - `yesterday` - Last Day

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            List of parameters.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[SubscriptionStatsQueryRecord]]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Statistic/customerbasic/{jsonable_encoder(mode)}/{jsonable_encoder(freq)}/{jsonable_encoder(customer_id)}",
            method="GET",
            params={
                "parameters": parameters,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[SubscriptionStatsQueryRecord],
                    parse_obj_as(
                        type_=typing.List[SubscriptionStatsQueryRecord],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def sub_stats(
        self,
        entry_id: int,
        interval: str,
        level: int,
        *,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.List[StatBasicQueryRecord]]:
        """
        Retrieves the subscription statistics for a given interval for a paypoint or organization.

        Parameters
        ----------
        entry_id : int
            Identifier in Payabli for the entity.

        interval : str
            Interval to get the data. Allowed values:

            - `all` - all intervals
            - `30` - 1-30 days
            - `60` - 31-60 days
            - `90` - 61-90 days
            - `plus` - +90 days

        level : int
            The entry level for the request:
              - 0 for Organization
              - 2 for Paypoint

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            List of parameters

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[StatBasicQueryRecord]]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Statistic/subscriptions/{jsonable_encoder(interval)}/{jsonable_encoder(level)}/{jsonable_encoder(entry_id)}",
            method="GET",
            params={
                "parameters": parameters,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[StatBasicQueryRecord],
                    parse_obj_as(
                        type_=typing.List[StatBasicQueryRecord],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def vendor_basic_stats(
        self,
        freq: str,
        id_vendor: int,
        mode: str,
        *,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.List[StatisticsVendorQueryRecord]]:
        """
        Retrieve the basic statistics about a vendor for a given time period, grouped by frequency.

        Parameters
        ----------
        freq : str
            Frequency to group series. Allowed values:

            - `m` - monthly
            - `w` - weekly
            - `d` - daily
            - `h` - hourly

            For example, `w` groups the results by week.

        id_vendor : int
            Vendor ID.

        mode : str
            Mode for request. Allowed values:

            - `ytd` - Year To Date
            - `mtd` - Month To Date
            - `wtd` - Week To Date
            - `today` - All current day
            - `m12` - Last 12 months
            - `d30` - Last 30 days
            - `h24` - Last 24 hours
            - `lasty` - Last Year
            - `lastm` - Last Month
            - `lastw` - Last Week
            - `yesterday` - Last Day

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            List of parameters

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[StatisticsVendorQueryRecord]]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Statistic/vendorbasic/{jsonable_encoder(mode)}/{jsonable_encoder(freq)}/{jsonable_encoder(id_vendor)}",
            method="GET",
            params={
                "parameters": parameters,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[StatisticsVendorQueryRecord],
                    parse_obj_as(
                        type_=typing.List[StatisticsVendorQueryRecord],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawStatisticClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def basic_stats(
        self,
        entry_id: int,
        freq: str,
        level: int,
        mode: str,
        *,
        end_date: typing.Optional[str] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        start_date: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.List[StatBasicQueryRecord]]:
        """
        Retrieves the basic statistics for an organization or a paypoint, for a given time period, grouped by a particular frequency.

        Parameters
        ----------
        entry_id : int
            Identifier in Payabli for the entity.

        freq : str
            Frequency to group series. Allowed values:

            - `m` - monthly
            - `w` - weekly
            - `d` - daily
            - `h` - hourly

            For example, `w` groups the results by week.

        level : int
            The entry level for the request:
              - 0 for Organization
              - 2 for Paypoint

        mode : str
            Mode for the request. Allowed values:

            - `custom` - Allows you to set a custom date range
            - `ytd` - Year To Date
            - `mtd` - Month To Date
            - `wtd` - Week To Date
            - `today` - All current day
            - `m12` - Last 12 months
            - `d30` - Last 30 days
            - `h24` - Last 24 hours
            - `lasty` - Last Year
            - `lastm` - Last Month
            - `lastw` - Last Week
            - `yesterday` - Last Day


        end_date : typing.Optional[str]
            Used with `custom` mode. The end date for the range.
            Valid formats:
              - YYYY-mm-dd
              - YYYY/mm/dd
              - mm-dd-YYYY
              - mm/dd/YYYY

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            List of parameters.

        start_date : typing.Optional[str]
            Used with `custom` mode. The start date for the range.
            Valid formats:
               - YYYY-mm-dd
               - YYYY/mm/dd
               -  mm-dd-YYYY
               - mm/dd/YYYY

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[StatBasicQueryRecord]]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Statistic/basic/{jsonable_encoder(mode)}/{jsonable_encoder(freq)}/{jsonable_encoder(level)}/{jsonable_encoder(entry_id)}",
            method="GET",
            params={
                "endDate": end_date,
                "parameters": parameters,
                "startDate": start_date,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[StatBasicQueryRecord],
                    parse_obj_as(
                        type_=typing.List[StatBasicQueryRecord],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def customer_basic_stats(
        self,
        customer_id: int,
        freq: str,
        mode: str,
        *,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.List[SubscriptionStatsQueryRecord]]:
        """
        Retrieves the basic statistics for a customer for a specific time period, grouped by a selected frequency.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        freq : str
            Frequency to group series. Allowed values:

            - `m` - monthly
            - `w` - weekly
            - `d` - daily
            - `h` - hourly

            For example, `w` groups the results by week.

        mode : str
            Mode for request. Allowed values:

            - `ytd` - Year To Date
            - `mtd` - Month To Date
            - `wtd` - Week To Date
            - `today` - All current day
            - `m12` - Last 12 months
            - `d30` - Last 30 days
            - `h24` - Last 24 hours
            - `lasty` - Last Year
            - `lastm` - Last Month
            - `lastw` - Last Week
            - `yesterday` - Last Day

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            List of parameters.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[SubscriptionStatsQueryRecord]]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Statistic/customerbasic/{jsonable_encoder(mode)}/{jsonable_encoder(freq)}/{jsonable_encoder(customer_id)}",
            method="GET",
            params={
                "parameters": parameters,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[SubscriptionStatsQueryRecord],
                    parse_obj_as(
                        type_=typing.List[SubscriptionStatsQueryRecord],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def sub_stats(
        self,
        entry_id: int,
        interval: str,
        level: int,
        *,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.List[StatBasicQueryRecord]]:
        """
        Retrieves the subscription statistics for a given interval for a paypoint or organization.

        Parameters
        ----------
        entry_id : int
            Identifier in Payabli for the entity.

        interval : str
            Interval to get the data. Allowed values:

            - `all` - all intervals
            - `30` - 1-30 days
            - `60` - 31-60 days
            - `90` - 61-90 days
            - `plus` - +90 days

        level : int
            The entry level for the request:
              - 0 for Organization
              - 2 for Paypoint

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            List of parameters

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[StatBasicQueryRecord]]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Statistic/subscriptions/{jsonable_encoder(interval)}/{jsonable_encoder(level)}/{jsonable_encoder(entry_id)}",
            method="GET",
            params={
                "parameters": parameters,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[StatBasicQueryRecord],
                    parse_obj_as(
                        type_=typing.List[StatBasicQueryRecord],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def vendor_basic_stats(
        self,
        freq: str,
        id_vendor: int,
        mode: str,
        *,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.List[StatisticsVendorQueryRecord]]:
        """
        Retrieve the basic statistics about a vendor for a given time period, grouped by frequency.

        Parameters
        ----------
        freq : str
            Frequency to group series. Allowed values:

            - `m` - monthly
            - `w` - weekly
            - `d` - daily
            - `h` - hourly

            For example, `w` groups the results by week.

        id_vendor : int
            Vendor ID.

        mode : str
            Mode for request. Allowed values:

            - `ytd` - Year To Date
            - `mtd` - Month To Date
            - `wtd` - Week To Date
            - `today` - All current day
            - `m12` - Last 12 months
            - `d30` - Last 30 days
            - `h24` - Last 24 hours
            - `lasty` - Last Year
            - `lastm` - Last Month
            - `lastw` - Last Week
            - `yesterday` - Last Day

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            List of parameters

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[StatisticsVendorQueryRecord]]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Statistic/vendorbasic/{jsonable_encoder(mode)}/{jsonable_encoder(freq)}/{jsonable_encoder(id_vendor)}",
            method="GET",
            params={
                "parameters": parameters,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[StatisticsVendorQueryRecord],
                    parse_obj_as(
                        type_=typing.List[StatisticsVendorQueryRecord],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

NotificationContentReportName = typing.Union[
    typing.Literal["Transaction", "Settlement", "Boarding", "Returned"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class PayabliCredentials(UniversalBaseModel):
    account_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="accountId")] = None
    cfee_fix: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="cfeeFix")] = None
    cfee_float: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="cfeeFloat")] = None
    cfee_max: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="cfeeMax")] = None
    cfee_min: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="cfeeMin")] = None
    maxticket: typing.Optional[float] = None
    minticket: typing.Optional[float] = None
    mode: typing.Optional[int] = None
    reference_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="referenceId")] = None
    service: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class VendorResponseStoredMethod(UniversalBaseModel):
    """
    Stored payment method information
    """

    id_pmethod: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="IdPmethod")] = None
    method: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Method")] = None
    descriptor: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Descriptor")] = None
    masked_account: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="MaskedAccount")] = None
    exp_date: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ExpDate")] = None
    holder_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="HolderName")] = None
    ach_sec_code: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="AchSecCode")] = None
    ach_holder_type: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="AchHolderType")] = None
    is_validated_ach: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="IsValidatedACH")] = None
    bin: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="BIN")] = None
    bin_data: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="binData")] = None
    aba: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ABA")] = None
    postal_code: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PostalCode")] = None
    method_type: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="MethodType")] = None
    last_updated: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="LastUpdated")] = None
    card_updated_on: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="CardUpdatedOn")] = (
        None
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .initiator import Initiator
from .pay_method_stored_method_method import PayMethodStoredMethodMethod
from .stored_method_usage_type import StoredMethodUsageType
from .storedmethodid import Storedmethodid


class PayMethodStoredMethod(UniversalBaseModel):
    """
    The required and recommended fields for a payment made with a stored payment method.
    """

    initiator: typing.Optional[Initiator] = None
    method: PayMethodStoredMethodMethod = pydantic.Field()
    """
    Method to use for the transaction. Use either `card` or `ach`, depending on what kind of method was tokenized to use a saved payment method for this transaction.
    """

    stored_method_id: typing_extensions.Annotated[
        typing.Optional[Storedmethodid], FieldMetadata(alias="storedMethodId")
    ] = pydantic.Field(default=None)
    """
    Payabli identifier of a tokenized payment method.
    """

    stored_method_usage_type: typing_extensions.Annotated[
        typing.Optional[StoredMethodUsageType], FieldMetadata(alias="storedMethodUsageType")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .auth_capture_payout_response import AuthCapturePayoutResponse
    from .auth_capture_payout_response_data import AuthCapturePayoutResponseData
    from .authorize_payment_method import AuthorizePaymentMethod
    from .authorize_payout_body import AuthorizePayoutBody
    from .capture_all_out_response import CaptureAllOutResponse
    from .capture_all_out_response_response_data_item import CaptureAllOutResponseResponseDataItem
    from .lot_number import LotNumber
    from .operation_result import OperationResult
    from .request_out_authorize_invoice_data import RequestOutAuthorizeInvoiceData
    from .request_out_authorize_payment_details import RequestOutAuthorizePaymentDetails
    from .request_out_authorize_vendor_billing_data import RequestOutAuthorizeVendorBillingData
    from .request_out_authorize_vendor_data import RequestOutAuthorizeVendorData
    from .request_out_authorize_vendor_data_old import RequestOutAuthorizeVendorDataOld
    from .v_card_get_response import VCardGetResponse
    from .v_card_get_response_associated_vendor import VCardGetResponseAssociatedVendor
    from .v_card_get_response_associated_vendor_billing_data import VCardGetResponseAssociatedVendorBillingData
    from .v_card_get_response_associated_vendor_summary import VCardGetResponseAssociatedVendorSummary
    from .v_card_get_response_contact import VCardGetResponseContact
_dynamic_imports: typing.Dict[str, str] = {
    "AuthCapturePayoutResponse": ".auth_capture_payout_response",
    "AuthCapturePayoutResponseData": ".auth_capture_payout_response_data",
    "AuthorizePaymentMethod": ".authorize_payment_method",
    "AuthorizePayoutBody": ".authorize_payout_body",
    "CaptureAllOutResponse": ".capture_all_out_response",
    "CaptureAllOutResponseResponseDataItem": ".capture_all_out_response_response_data_item",
    "LotNumber": ".lot_number",
    "OperationResult": ".operation_result",
    "RequestOutAuthorizeInvoiceData": ".request_out_authorize_invoice_data",
    "RequestOutAuthorizePaymentDetails": ".request_out_authorize_payment_details",
    "RequestOutAuthorizeVendorBillingData": ".request_out_authorize_vendor_billing_data",
    "RequestOutAuthorizeVendorData": ".request_out_authorize_vendor_data",
    "RequestOutAuthorizeVendorDataOld": ".request_out_authorize_vendor_data_old",
    "VCardGetResponse": ".v_card_get_response",
    "VCardGetResponseAssociatedVendor": ".v_card_get_response_associated_vendor",
    "VCardGetResponseAssociatedVendorBillingData": ".v_card_get_response_associated_vendor_billing_data",
    "VCardGetResponseAssociatedVendorSummary": ".v_card_get_response_associated_vendor_summary",
    "VCardGetResponseContact": ".v_card_get_response_contact",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AuthCapturePayoutResponse",
    "AuthCapturePayoutResponseData",
    "AuthorizePaymentMethod",
    "AuthorizePayoutBody",
    "CaptureAllOutResponse",
    "CaptureAllOutResponseResponseDataItem",
    "LotNumber",
    "OperationResult",
    "RequestOutAuthorizeInvoiceData",
    "RequestOutAuthorizePaymentDetails",
    "RequestOutAuthorizeVendorBillingData",
    "RequestOutAuthorizeVendorData",
    "RequestOutAuthorizeVendorDataOld",
    "VCardGetResponse",
    "VCardGetResponseAssociatedVendor",
    "VCardGetResponseAssociatedVendorBillingData",
    "VCardGetResponseAssociatedVendorSummary",
    "VCardGetResponseContact",
]
